﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Irony.Parsing;
using Irony.Interpreter.Ast;
using Irony.Interpreter;

namespace Fi.Partners.Plugins.Traffic.TimeArithmetic
{
	public class TimeCalculatorGrammar : InterpretedLanguageGrammar
	{
		public TimeCalculatorGrammar()
			: base(false)
		{
			//Terminals
			var number = new NumberLiteral("number", NumberOptions.AllowLetterAfter);
			var HOUR_SINGLE = ToTerm("h");
			var HOUR_SHORT = ToTerm("hr");
			var HOUR_LONG = ToTerm("hour");
			var HOURS_SHORT = ToTerm("hrs");
			var HOURS_LONG = ToTerm("hours");

			var MINUTE_SINGLE = ToTerm("m");
			var MINUTE_SHORT = ToTerm("min");
			var MINUTE_LONG = ToTerm("minute");
			var MINUTES_SHORT = ToTerm("mins");
			var MINUTES_LONG = ToTerm("minutes");

			var COLON = ToTerm(":");

			var PLUS = ToTerm("+");
			var MINUS = ToTerm("-");

			//Non-terminals
			var hourSymbol = new NonTerminal("hourSymbol");
			var minuteSymbol = new NonTerminal("minuteSymbol");
			var hourValue = new NonTerminal("hourValue", typeof(HourValueNode));
			var minuteValue = new NonTerminal("minuteValue", typeof(MinuteValueNode));
			var colonTimeValue = new NonTerminal("colonTimeValue", typeof(ColonTimeValueNode));
			var hourMinuteTimeValue = new NonTerminal("hourMinuteTimeValue", typeof(HourMinuteTimeValueNode));
			var binExpr = new NonTerminal("binExpr", typeof(BinaryOperationNode));
			var binOperator = new NonTerminal("binOperator");
			var unExpr = new NonTerminal("unExpr", typeof(UnaryOperationNode));
			var unOperator = new NonTerminal("unOperator");
			var timeValue = new NonTerminal("timeValue");
			var expression = new NonTerminal("expression", typeof(ExpressionNode));
			var timePartSeparatorSymbol = new NonTerminal("timePartSeparatorSymbol");

			hourSymbol.Rule = HOUR_SINGLE | HOUR_SHORT | HOUR_LONG | HOURS_SHORT | HOURS_LONG;
			hourValue.Rule = number + hourSymbol;
			minuteSymbol.Rule = MINUTE_SINGLE | MINUTE_SHORT | MINUTE_LONG | MINUTES_SHORT | MINUTES_LONG;
			minuteValue.Rule = number + minuteSymbol;
			colonTimeValue.Rule = number + COLON + number;
			hourMinuteTimeValue.Rule = hourValue + minuteValue;

			timeValue.Rule = hourValue | minuteValue | colonTimeValue | hourMinuteTimeValue;
			expression.Rule = timeValue | binExpr | unExpr;
			binOperator.Rule = PLUS | MINUS;
			binExpr.Rule = expression + binOperator + expression;
			unOperator.Rule = PLUS | MINUS;
			unExpr.Rule = binOperator + timeValue;

			//Operators
			RegisterOperators(10, "+", "-");
			MarkTransient(hourSymbol, minuteSymbol, timeValue, binOperator, unOperator);

			//binOperator.SetFlag(TermFlags.InheritPrecedence); 

			this.Root = expression;
		}
	}
}
